function sdkp:ChatCommand(input)
    if not input or input:trim() == "" then
        InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
    else
        LibStub("AceConfigCmd-3.0").HandleCommand(sdkp, "sdkp", "sdkp", input)
    end
end

-- Purge handler
function sdkp:Purge(info)
    self.db.profile.snapshots = {};
    self.db.profile.loot = {};
    self.db.profile.bids = {};
    self:Print("Snapshot and loot data purged.");
	self.BIDSCROLLFRAME:SortData();
	self.ENCOUNTERLIST_SCROLLFRAME:SetData(self.db.profile.snapshots);
end

-- Snapshot handler
function sdkp:CmdSnapshot(info, notes, loot)
	self:Debug("Taking a snapshot...");
	local snapshots = self.db.profile.snapshots;
	notes = strtrim(notes or "");
	
	if not sdkp:IsTrackLoot() then 
		sdkp:Debug("Loot tracking is not on, skipping snapshot.");
		return;
	elseif notes == "" then -- manual snapshot, allow even if not tracking loot. 
		notes = GetRealZoneText() .. " - Manual Snapshot";
	end
	
	local thissnap = snapshots[#snapshots];
	if not(thissnap) or thissnap.cols[1].value ~= notes then 
		-- take the snapshot
		table.insert(snapshots, {
			["cols"] = {
				{ ["value"] = notes, }, -- [1]
				{ ["value"] = sdkp.GetCloseButtonTexture, }, -- [2]
			},
			["playertable"] = {},
			["timestamp"] = time(),
			["zone"] = GetRealZoneText(),
		});
		thissnap = snapshots[#snapshots];
		self:Debug("Snapshot inserted into snapshots table.");
		
		local nRaidMembers = GetNumRaidMembers();
		self.takingsnapshot = true;
		for raidIndex = 1, nRaidMembers do
			local raidMember = GetRaidRosterInfo(raidIndex);
			local main = self:GetAltInfo(raidMember);
			if main then 
				raidMember = main;	
			end
			self:CmdHandleManualSnapshot(raidMember, "add", thissnap)
		end
		self.takingsnapshot = nil;
		
		self:Debug("Shout out to the homies...");
		self:SendGuild("["..notes.."] BOSS DOWN!!!");
		if self:IsAnnounceLoot() and loot then
			local count = 1;
			local message = ""; 
			for item in pairs(loot) do 
				message = message .. item;
				count = count + 1;
				if count == 3 then 
					self:SendGuild(message);
					message = "";
					count = 0;
				end
			end
			if message ~= "" then 
				self:SendGuild(message);
			end
		end
		if self:IsAttCheck() then 
			self:SendGuild("ATTENDANCE CHECK! Response needed from NON raid members within ".. string.format("%.2f", self:GetAttCheckLen()/60) .. " min.");
			self:SendGuild("In order to get credit for this boss, you must [  /w " .. UnitName("player") .. " !here ]");
			self:ScheduleTimer(function() 
				self.checkingatt = false;
				self:SendGuild("ATTENDANCE CHECK HAS ENDED!  That is all...");
			end, self:GetAttCheckLen(), nil);
			self.checkingatt = true;
		end
	end
	
	self.ENCOUNTERLIST_SCROLLFRAME:SortData();
	sdkp.ENCOUNTERDETAIL_SCROLLFRAME:SortData();
end

-- print debug mode handlers
function sdkp:IsDebug(info)
    return self.db.profile.debug;
end
function sdkp:ToggleDebug(info, value)
    self.db.profile.debug = value;
    SDKP_RAID_ZONE["Scarlet Monastery"] = self.db.profile.debug; -- easy to test on sm bosses, close to zone.
end

-- Loot handlers
function sdkp:IsAnnounceLoot(info, key)
    return self.db.profile.announceloot;
end
function sdkp:ToggleAnnounceLoot(info,  value)
	self.db.profile.announceloot = value;
end
function sdkp:IsTrackLoot(info)
    return self.db.profile.trackloot;
end
function sdkp:ToggleTrackLoot(info, value)
    self.db.profile.trackloot = value;
end
function sdkp:GetLootFilter(info, key)
    return self.db.profile.lootfilter[SDKP_COLORS[key]];
end
function sdkp:SetLootFilter(info, key, value)
	self.db.profile.lootfilter[SDKP_COLORS[key]] = value;
end

-- Timers
function sdkp:GetAttCheckLen(info)
    return self.db.profile.attchecklen;
end
function sdkp:SetAttCheckLen(info, value)
    self.db.profile.attchecklen = value;
end
function sdkp:GetSanctionLen(info)
    return self.db.profile.sanctionlen;
end
function sdkp:SetSanctionLen(info, value)
    self.db.profile.sanctionlen = value;
end

-- Dkp system handlers
function sdkp:IsAttCheck(info)
    return self.db.profile.checkattendance;
end
function sdkp:ToggleAttCheck(info, value)
    self.db.profile.checkattendance = value;
end
function sdkp:GetTaxThreshold(info)
    return self.db.profile.taxthreshold;
end
function sdkp:SetTaxThreshold(info, value)
    self.db.profile.taxthreshold = value;
end
function sdkp:GetTaxRate(info)
    return self.db.profile.taxrate;
end
function sdkp:SetTaxRate(info, value)
    self.db.profile.taxrate = value;
end
function sdkp:IsRelationalDkp(info)
    return self.db.profile.relationaldkp;
end
function sdkp:ToggleRelationalDkp(info, value)
    self.db.profile.relationaldkp = value;
	self:RefreshScrollTables();
end
function sdkp:GetInitialGearValue(info)
    return self.db.profile.initialgearvalue;
end
function sdkp:SetInitialGearValue(info, value)
	self.db.profile.initialgearvalue = value;
	self:RefreshScrollTables();
end

-- Ignore loot handlers
function sdkp:CmdIgnoreLoot(info, link)
	if link == "" then 
		self:Print("|cffffff78Usage:|r /sdkp ignoreloot [Item]    |cffffff78Example:|r /sdkp il |cff0070dd|Hitem:32428:0:0:0:0:0:0:0|h[Heart of Darkness]|h|r");
		self:Print(self:ListIgnoreLoot());
		return;
	end
	local _, _, item = string.find(link, "|c%x+|Hitem:%d+[:%d%-]+|h%[(.-)%]|h|r");
	if not item then 
		item = link;
	end		 
	if self.db.profile.ignoreloot[item] then
		self.db.profile.ignoreloot[item] = nil;
		self:Print("No longer ignoring " .. link);
	else
		self.db.profile.ignoreloot[item] = true;
		self:Print("Now ignoring " .. link);
	end
end
function sdkp:IsIgnoreLoot(item)
	return self.db.profile.ignoreloot[item] == true;
end
function sdkp:ListIgnoreLoot()
	local count = 0;
	for key, value in pairs(self.db.profile.ignoreloot) do
		count = count + 1;
	end
	return "Currently ignoring " .. count .. " items.";
end

-- Assign loot handlers
function sdkp:CmdAssignLoot(info, args)
	if args == "" then 
		self:Print("|cffffff78Usage:|r /sdkp assignloot [Item] player (reason/note)");
		self:Print("|cffffff78Example:|r /sdkp al |cff0070dd|Hitem:32428:0:0:0:0:0:0:0|h[Heart of Darkness]|h|r Sachmo");
		self:Print("|cffffff78Example:|r /sdkp al |cff0070dd|Hitem:32428:0:0:0:0:0:0:0|h[Heart of Darkness]|h|r Sachmo because he is awesome!");
		return;
	end
	self:Debug("sdkp:CmdAssignLoot", info, args);
	string.gsub(args, "%s*(|c%x+|Hitem:%d+[:%d%-]+|h%[.-%]|h|r)%s+([^%s]+)%s*(.*)", function(link, player, note)
		if not note or note == "" then 
			note = "Manually assigned";
		end
		self:Debug("sdkp:CmdAssignLoot GSUB", link, player, note);
		self:CHAT_MSG_LOOT(note, player .. " receives loot: " .. link .. ".");
		return false;
	end);
end

-- Manual attendance/dkp snapshots handler
function sdkp:CmdHandleManualSnapshot(player, command, snapshot)
	if not player or strtrim(player) == "" then
		return;
	end
	player = strtrim(player);
	local main = self:GetAltInfo(player);
	if main then 
		player = main;
	end
	local message;
	
	for i, row in ipairs(snapshot["playertable"]) do 
		if row.cols[1].value == player then
			if command == "add" then
				message = player .. " was already in the snapshot.";
			elseif command == "remove" then 
				table.remove(snapshot, i);
				message = player .. " has been removed from the snapshot.";
			end
			if not self.takingsnapshot then 
				self:Print(message);
			end
			return;
		end
	end
	if command == "add" then 
		table.insert(snapshot["playertable"], {
			["cols"] = {
				{
					["value"] = player,
					["color"] = sdkp.EncounterDetailNameColor,
				} -- [1]
			},
		});
		message = player .. " added to the snapshot.";
	else 
		message = player .. " was not in the snapshot.";
	end
	if not self.takingsnapshot then 
		self:Print(message);
	end
 
	self:RefreshScrollTables();
end

-- Manually assign major/minor noobery to a player
function sdkp:CmdNoobery(type, args)
	if not args or strtrim(args) == "" then
		return;
	end
	local _, _, player, note = args:find("([^%s]+)%s*(.*)");
	if not note or note == "" then
		note = "Just Because...";
	end
	local main = self:GetAltInfo(player);
	if main then player = main; end 
	
	local which, price = "Minor Noobery", 1;
	if type == "major" then 
		which, price = "Major Noobery", 2;
	end
	local message = player .. " receives loot: " .. which .. ". (" .. note .. ")";
	self:SendOfficer(message);
	self:SendRaid(message);
	
	table.insert(self.db.profile.loot, {
		["player"] = player, 
		["item"] = which, 
		["price"] = price, 
		["id"] = "", 
		["timestamp"] = time(), 
		["note"] = note,
		["color"] = "ffff8000",
	});
	self:RefreshScrollTables();
end

-- Export handler
function sdkp:CmdExport(info)
	if not self.db.profile.snapshots or #self.db.profile.snapshots < 1 then 
		self:Print("No snapshot data!");
		return;
	end
	local ts = self.db.profile.snapshots[1].timestamp;
	local time = date("%m/%d/%y %H:%M:%S", ts);
	
	local exportdata = "<RaidInfo><key>"..time.."</key><start>"..time.."</start><end>"..time.."</end><zone></zone>";
	exportdata = exportdata.."<BossKills></BossKills><note>" .. self.db.profile.snapshots[1].zone .. "</note>";
	
	local joinleave = "";
	local count = 1;
	for i, snapshot in ipairs(self.db.profile.snapshots) do
		time = date("%m/%d/%y %H:%M:%S", ts+i);
		for j, playerentry in pairs(snapshot.playertable) do 
			joinleave = joinleave.."<key"..count.."><player>"..playerentry.cols[1].value.."</player><time>"..time.."</time></key"..count..">";
			count = count + 1;
		end
	end
	exportdata = exportdata.."<Join>"..joinleave.."</Join><Leave>"..joinleave.."</Leave><Loot>";
	
	local i, j = #self.db.profile.loot, #self.db.profile.snapshots;
	local boss;
	while i >= 1 do
		local itemdata = self.db.profile.loot[i];
		exportdata = exportdata.."<key"..i..">";
		exportdata = exportdata.."<ItemName>"..itemdata.item.."</ItemName>";
		exportdata = exportdata.."<ItemID>"..itemdata.id.."</ItemID>";
		exportdata = exportdata.."<Icon></Icon><Class></Class><SubClass></SubClass>";
		exportdata = exportdata.."<Color>"..itemdata.color.."</Color>";
		exportdata = exportdata.."<Count>1</Count>";
		exportdata = exportdata.."<Player>"..itemdata.player.."</Player>";
		
		while j >= 1 do
			if itemdata.timestamp > self.db.profile.snapshots[j].timestamp then 
				time = date("%m/%d/%y %H:%M:%S", ts+j);
				boss = self.db.profile.snapshots[j].cols[1].value;
				break;
			end		
			j = j - 1;	
		end
		if j == 0 then 
			time = date("%m/%d/%y %H:%M:%S", ts+1);
			boss = self.db.profile.snapshots[1].cols[1].value;
		end
		
		exportdata = exportdata.."<Time>"..time.."</Time>";
		exportdata = exportdata.."<Note><![CDATA[ - Boss: "..boss.." - "..itemdata.price.." DKP]]></Note>";
		exportdata = exportdata.."</key"..i..">";
		i = i - 1;
	end
	exportdata = exportdata.."</Loot></RaidInfo>";
	
	self:EnqueueDialog(
		"CTRA DKP Export",
		"Ok", 
		"Cancel", 
		function()end,
		function()end,
		exportdata
	); 
	self:ConfirmPopup();
end

-- sync handler
function sdkp:CmdSync(info, args)
	if not args or strtrim(args) == "" then
		self:Print("Who to sync with?");
		return;
	end
	local _, _, who = args:find("([^%s]+)");
	self.requestsync = who;
	self:SendCommMessage(self.name, self:Serialize("requestsync"), "WHISPER", who, "BULK");
end
